@@ -219,8 +219,8 @@ case "$lisp" in
       allegromodern8_64_s) command="${ALLEGROMODERN8_64_S:-mlisp8_64_s}" ;;
     esac
     # For the sake of the lisp-invocation library, re-export these
-    ALLEGRO=$command ; export ALLEGRO ;
-    echo ALLEGRO=$ALLEGRO
+    # ALLEGRO=$command ; export ALLEGRO ;
+    # echo ALLEGRO=$ALLEGRO
     flags="-q"
     nodebug="-batch"
     if [ "$os" = windows ] && [ -z "$ALLEGRO_NOISY" ] ; then
@@ -284,7 +284,8 @@ case "$lisp" in
     eval="-eval" ;;
   sbcl)
     command="${SBCL:-sbcl}"
-    flags="--noinform --no-userinit --no-sysinit"
+    flags="--no-userinit --no-sysinit"
+    # flags="--noinform --no-userinit --no-sysinit"
     nodebug="--disable-debugger"
     eval="--eval" ;;
   scl)
@@ -501,7 +502,7 @@ test_clean_load () {
       "(or'#.(load(string'|test/script-support.lisp|):verbose():print())#.(asdf-test:exit-lisp'0))" \
       > $nop 2>&1
     $bcmd $eval \
-      "(or'#.(load(string'|test/script-support.lisp|):verbose():print())#.(asdf-test:verbose())#.(load(string'|build/asdf.lisp|):verbose())#.(asdf/image:quit'0))" \
+      "(or'#.(load(string'|test/script-support.lisp|):verbose():print())#.(asdf-test:verbose())#.(load(string'|build/asdf.lisp|):verbose())#.(uiop/image:quit'0))" \
       > $load 2>&1
     if diff $nop $load ; then
       echo "GOOD: Loading ASDF on $lisp produces no message" >&2 ; return 0
