@@ -35,16 +35,17 @@ Some constraints:
 
 (in-package :asdf-test)
 
+#+(and ecl (not ecl-bytecmp)) (require :cmp)
+
 (declaim (optimize (speed 2) (safety #-gcl 3 #+gcl 0) #-(or allegro gcl genera) (debug 3)
                    #+(or cmu scl) (c::brevity 2)))
 (proclaim '(optimize (speed #-gcl 2 #+gcl 1) (safety #-gcl 3 #+gcl 0) #-(or allegro gcl genera) (debug 3)
                      #+(or cmu scl) (c::brevity 2) #+(or cmu scl) (ext:inhibit-warnings 3)))
 
-(defvar *trace-symbols*
+(defparameter *trace-symbols*
   `(;; If you want to trace some stuff while debugging ASDF,
     ;; here's a nice place to say what.
     ;; These string designators will be interned in ASDF after it is loaded.
-
     ;;#+ecl ,@'( :perform :input-files :output-files :compile-file* :compile-file-pathname* :load*)
     ))
 
@@ -204,9 +205,41 @@ Some constraints:
 ;; We still want to work despite and host/device funkiness,
 ;; so we do it the hard way.
 (defparameter *test-directory*
-  (truename
-   (make-pathname :name nil :type nil :version nil
-                  :defaults (or *load-pathname* *compile-file-pathname* *default-pathname-defaults*))))
+  ;; the following is annoyingly named "test-getenv" to avoid symbol clash
+  ;; later when we use UIOP.
+  (flet ((test-getenv (x)
+           (declare (ignorable x))
+           #+(or abcl clasp clisp ecl xcl) (ext:getenv x)
+           #+allegro (sys:getenv x)
+           #+clozure (ccl:getenv x)
+           #+cmucl (unix:unix-getenv x)
+           #+scl (cdr (assoc x ext:*environment-list* :test #'string=))
+           #+cormanlisp
+           (let* ((buffer (ct:malloc 1))
+                  (cname (ct:lisp-string-to-c-string x))
+                  (needed-size (win:getenvironmentvariable cname buffer 0))
+                  (buffer1 (ct:malloc (1+ needed-size))))
+             (prog1 (if (zerop (win:getenvironmentvariable cname buffer1 needed-size))
+                        nil
+                      (ct:c-string-to-lisp-string buffer1))
+               (ct:free buffer)
+               (ct:free buffer1)))
+           #+gcl (system:getenv x)
+           #+genera nil
+           #+lispworks (lispworks:environment-variable x)
+           #+mcl (ccl:with-cstrs ((name x))
+                                 (let ((value (_getenv name)))
+                                   (unless (ccl:%null-ptr-p value)
+                                     (ccl:%get-cstring value))))
+           #+mkcl (#.(or (find-symbol "GETENV" :si) (find-symbol "GETENV" :mk-ext)) x)
+           #+sbcl (sb-ext:posix-getenv x)
+           #-(or abcl allegro clasp clisp clozure cmucl cormanlisp ecl gcl genera lispworks mcl mkcl sbcl scl xcl)
+           (error "~S is not supported on your implementation" 'getenv)))
+    (or (test-getenv "ASDF_TEST_DIRECTORY")
+        (truename
+         (make-pathname :name nil :type nil :version nil
+                        :defaults (or *load-pathname* *compile-file-pathname* *default-pathname-defaults*))))))
+
 (defun make-sub-pathname (&rest keys &key defaults &allow-other-keys)
   (merge-pathnames (apply 'make-pathname keys) defaults))
 (defparameter *asdf-directory*
@@ -426,7 +459,7 @@ is bound, write a message and exit on an error.  If
 (defmacro with-asdf-conditions ((&optional verbose) &body body)
   `(call-with-asdf-conditions #'(lambda () ,@body) ,verbose))
 
-(defun compile-asdf (&optional tag verbose upgradep)
+(defun compile-asdf (&optional tag (verbose nil) upgradep)
   (let* ((alisp (asdf-lisp tag))
          (afasl (asdf-fasl tag))
          (tmp (make-pathname :name "asdf-tmp" :defaults afasl)))
@@ -549,6 +582,13 @@ is bound, write a message and exit on an error.  If
   (load-asdf-lisp)
   (clean-asdf-system))
 
+(defun hash-table-alist (h)
+  (loop for k being the hash-keys of h using (hash-value v) collect (cons k v)))
+
+(defun registry ()
+  (let ((sr (asymval :*source-registry*)))
+    (when sr (sort (hash-table-alist sr) 'string< :key 'car))))
+
 (defun configure-asdf ()
   (format t "Configuring ASDF~%")
   (when (asym :getenvp)
